"""Create and manage layout
"""

import gtk
import gobject
import pango
import Callbacks

class Interface:
    """Create and manage layout
    @author: Bartosz 'Perfekcik' Alchimowicz
    """

    def __init__(self, pluginDB, callbacks):
	self.__window = None
	self.__buffer = None
	self.__treeView = None
	self.__PLUGIN_NAME = 0
	self.__PLUGIN_NUMBER = 1
	self.__toFocus = None
	self.__pluginDB = pluginDB
	self.__callbacks = callbacks;

    # layout
    
    def __createWindow(self):
	"""Create window and set title.\n
	Connect callbacks for destroy and delete_event.
	"""
    
	window = gtk.Window(gtk.WINDOW_TOPLEVEL)
	window.set_title('Aurox Config Center')
	window.set_default_size(640, 480)
	
	window.connect('destroy', self.__callbacks.destroy)
	window.connect('delete_event', self.__callbacks.delete_event)
	return window

    def __createMainBox(self, parent):
	vbox = gtk.VBox(gtk.FALSE, 0)
	parent.add(vbox)
	vbox.show()
	return vbox

    def __createImage(self, parent):
	image = gtk.Image()
	image.set_from_file('../acc.png')
	image.set_size_request(-1, 120)
	image.show()

	hbox = gtk.HBox(gtk.FALSE, 0)
	hbox.pack_start(image, gtk.FALSE, gtk.FALSE, 22)
	hbox.show()

	parent.pack_start(hbox, gtk.FALSE, gtk.FALSE, 0)

	return image

    def __createVPaned(self, parent):
	vpaned = gtk.HPaned()
	parent.pack_start(vpaned)
	vpaned.show()
	return vpaned

    def __createVPanedLeftSide(self, parent):
    	leftSide = gtk.ScrolledWindow()
	leftSide.set_size_request(200, -1)
	leftSide.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
	parent.add1(leftSide)
	leftSide.show()
	return leftSide

    def __createVPanedRightSide(self, parent):
	rightSide = gtk.ScrolledWindow()
	rightSide.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
	parent.add2(rightSide)
	rightSide.show()
	return rightSide

    def __createTreeView(self, parent):
	"""Create ListStore in TreeModelSort and put into TreeView
	"""
	listStore = gtk.ListStore(gobject.TYPE_STRING, int)

	treeModelSort = gtk.TreeModelSort(listStore)
	treeModelSort.set_sort_column_id(0, gtk.SORT_ASCENDING)
	
	# treeView = gtk.TreeView(listStore)
	treeView = gtk.TreeView(treeModelSort)
	treeView.connect('row-activated', self.__callbacks.rowDBClick)
	
	selection = treeView.get_selection()
	selection.set_mode(gtk.SELECTION_SINGLE)
	selection.connect('changed', self.__callbacks.rowClick, self)

	cell = gtk.CellRendererText()
	treeViewColumn = gtk.TreeViewColumn('Plugins:', cell, 
		text = self.__PLUGIN_NAME)
		
	treeView.append_column(treeViewColumn)
	
	parent.add(treeView)
	treeView.show()
	
	treeView.set_headers_visible(gtk.FALSE)

	return treeView

    def __createTextView(self, parent):
	textView = gtk.TextView()
	textView.set_editable(gtk.FALSE)
	textView.set_cursor_visible(gtk.FALSE)
	textView.set_wrap_mode(gtk.WRAP_WORD)
	parent.add(textView)
	textView.show()
	return textView.get_buffer()

    def __createButtons(self, parent):
    	hbox = gtk.HBox(gtk.FALSE, 0)
	hbox.set_size_request(-1, 30)
	
	buttonAbout = gtk.Button(_('About'))
	buttonAbout.set_size_request(95, -1)
	buttonAbout.connect('clicked', self.__callbacks.about, self)
	buttonRun = gtk.Button(_('Run'))
	buttonRun.connect('clicked', self.__callbacks.run, self.__treeView)
	buttonRun.set_size_request(95, -1)
	buttonQuit = gtk.Button(_('Quit'))
	buttonQuit.set_size_request(95, -1)
	buttonQuit.connect('clicked', self.__callbacks.destroy)
	
	self.__toFocus = buttonQuit
	
	hbox.pack_start(buttonAbout, gtk.FALSE, gtk.FALSE, 10)
	buttonAbout.show()
	hbox.pack_end(buttonQuit, gtk.FALSE, gtk.FALSE, 10)
	buttonQuit.show()
	hbox.pack_end(buttonRun, gtk.FALSE, gtk.FALSE, 10)
	buttonRun.show()

	parent.pack_start(hbox, gtk.FALSE, gtk.FALSE, 10)

	hbox.show()

	return hbox


    def __addPlugins(self):
	# listStore = self.__treeView.get_model()
	treeModelSort = self.__treeView.get_model().get_model()
    
#	???????????????????????????????????????        
#	for number, plugin in enumerate(pluginDB):
#	    data = [plugin.getName(), number]
#	    listStore.append(data)

	number = 0

	for plugin in self.__pluginDB:
	    data = [plugin.getName(), number]
	    treeModelSort.append(data)
	    number = number + 1
	    
	return


    # layout
    def createWindow(self):
	self.__window = self.__createWindow()
	vBox = self.__createMainBox(self.__window)
	image = self.__createImage(vBox)	
	vPaned = self.__createVPaned(vBox)
	vPanedLeftSide = self.__createVPanedLeftSide(vPaned)
	self.__treeView = self.__createTreeView(vPanedLeftSide) 
	vPanedRightSide = self.__createVPanedRightSide(vPaned)
	self.__buffer = self.__createTextView(vPanedRightSide)
	self.__createButtons(vBox)
	
	self.__addPlugins()
	
	
	self.__toFocus.grab_focus()

#	self.__treeView.grab_focus()
#	self.__treeView.set_cursor(0)
	
    def showWindow(self):
	self.__window.show()
	gtk.main()
	
	
    # setters/getters
    def setText(self, header, text):
	"""@type header: str
	@param header: this text will by displayed in top of window 
	as pango.SCALE_XX_LARGE
	@type text: str
	@param text: rest of message
	"""
    
	startIter = self.__buffer.get_start_iter()
	endIter = self.__buffer.get_end_iter()
	buffer = self.__buffer
	
	buffer.remove_all_tags(startIter, endIter)

	buffer.set_text(header + '\n')
	endIter = buffer.get_end_iter()    
	buffer.insert(endIter, text)
	
	tagHeader = buffer.create_tag(scale = pango.SCALE_XX_LARGE, 
		pixels_below_lines = 10)
		
	startIter = buffer.get_start_iter()
	endIter = buffer.get_iter_at_line(1)
	
	buffer.apply_tag(tagHeader, startIter, endIter)
