# -*- coding: UTF-8 -*-
"""Colection of methods to create database of avaliable plugins.
"""

import os
import sys
import Plugin
import re

class PluginDataBase:
    """Colection of methods to create database of avaliable plugins.
    @author: Bartosz 'Perfekcik' Alchimowicz
    """
    def __getDirectoryList(self):
	"""
	@rtype: List
	@return: files and diretories in plugins directory
	"""
    	return os.listdir(os.getcwd())

    def __readConfigFile(self, dirName):
	"""Read config file (acc.cfg).
	@type dirName: str
	@param dirName: directory localization
	@rtype List
	@return list os entrys in file
	"""
        try:
	    data = open(dirName + '/acc.cfg', 'r').read().split('\n')
	except:
	    print (_('ERROR:') + ' ' + _('no acc.cfg file in %s')) \
		    .encode(_('iso-8859-1')) \
		    % dirName
	    pass
    
	return data

    def __createExe(self, dirName, exe):
        """Check if data is poining to absolut path and 
	if exe file is readable and executable.
	@type dirName: str
	@param dirName: directory localization(name of directory in plugins)
	@type exe: str
	@param exe: program file name
	@rtype str	
	@return absolut path to filr
	"""
	# print dirName
	# print exe
		
	error = 0
        tmpExe = os.path.join(dirName, exe)
	
	# print tmpExe
	
	if os.path.isfile(tmpExe):
	    if not os.access(tmpExe, os.X_OK|os.R_OK):
                print (_('ERROR:') + ' ' + 
			_('%s is not readable or not executable')) \
			.encode(_('iso-8859-1')) \
                	% (exe)
        	raise Exception
	    		    
	    exe = tmpExe
	else:
    	    whichCommand = 'which ' + exe + ' &>/dev/null'
	    whichStatus =  os.system(whichCommand)
	    
	    if (whichStatus):
                print (_('ERROR:') + ' ' + 
			_('can not find %s')) \
			.encode(_('iso-8859-1')) \
                	% (exe)
        	raise Exception
		
	return exe
	    	    
    def __createComment(self, dirName, comment):
        """Check if data is poining to absolut path and
	if exe file is readable
	@type dirName: str
	@param dirName: directory localization
	@type comment: str
	@param comment: file name
	"""
	
	if not os.path.isfile(dirName + '/' + comment):
	    return comment
	
	comment = dirName + '/' + comment
	
        if not os.access(comment, os.R_OK):
	    print (_('ERROR:') + ' ' 
		    + _('can not find description file %s')) \
		    .encode(_('iso-8859-1')) \
		    % (comment)
	    raise Exception
	
	return comment

    def __selectData(self, data):
	"""Analize data.
	@type data: str
	@param data: information to analize
	@rtype: tuple
	@return: name to display, comment file localization, file name localization
	"""
	langAll = os.environ['LANG'].lower()
	langFirst = langAll.split('_')[0]
    
    	name = ['', '']
	comment = ['', '']
	exe = ""

	nameRE = re.compile('name(\[((\w{2})(_\w{2})?)\])?');
	commentRE = re.compile('comment(\[((\w{2})(_\w{2})?)\])?');

	for line in data:
	    if line == '':
	        break

	    line = line.split('=')

	    result = nameRE.match(line[0].lower())
	    if result:
		if result.group(0) == 'name':
		    name[0] = line[1]
	    			    
		try:
		    if result.group(2) == langFirst:
			if name[1] == '':
			    name[1] = line[1]
		    else:
    			if result.group(2) == langAll:
			    name[1] = line[1]
		except:
		    pass

	    result = commentRE.match(line[0].lower())
	    if result:
		if result.group(0) == 'comment':
		    comment[0] = line[1]
	    			    
		try:
		    if result.group(2) == langFirst:
			if comment[1] == '':
			    comment[1] = line[1]
		    else:
    			if result.group(2) == langAll:
			    comment[1] = line[1]
		except:
		    pass
	    
	    if line[0].lower() == 'exec':
	        exe = line[1]
	
	#choose information for visualization
	if name[0] == '':
	    raise Exception

	if comment[0] == '':
	    raise Exception
	
        if name[1] != '':
	    name[0] = name[1]

        if comment[1] != '':
	    comment[0] = comment[1]
	
	return name[0], comment[0], exe
        
    def __createPluginList(self, dirList):
	"""Create plugin list. 
	@type dirList: List
	@param dirList: all avaliable data in directory
	@rtype: List
	@return: correct list of avaliable plugins, 
	this list is construced of Plugin objects
	"""
    	pluginDB = []
		
	for dirName in dirList:
    	    try:
		data = self.__readConfigFile(dirName)
		name, comment, exe  = self.__selectData(data)
		exe = self.__createExe(dirName, exe)
		comment = self.__createComment(dirName, comment)
	    except:
		continue

	    plugin = Plugin.Plugin(name, exe, comment)
		    
	    pluginDB.append(plugin)
	    
	return pluginDB
		
    def getPluginDataBase(self):
	"""Run this methods to recive database.\n
	First read all avaliable plugins, then retrive information from 
	acc.cfg files. There is a checking, if data in those files are correct.
	"""
	os.chdir(os.path.dirname(__file__) + '/../plugins')
	
	dirList = self.__getDirectoryList()
	pluginDB = self.__createPluginList(dirList)

	if len(pluginDB) == 0:
	    print (('ERROR: ') + ('Zero plugins loaded!'));

	return pluginDB
